//  HDMI SELECTOR controll
//　AVR　ATmega88用
//　Programming　By　亀屋　2018/1/03~
//  v1.0.0 2018/1/5　1st Release
//  

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/wdt.h>

// Sub routin
void countup(void);

// CPU freq.
#define FOSC 1000000 	// 1MHz internal CPU clock

//on RAM
volatile uint8_t SW_in;
volatile uint8_t SW_jdg;
volatile uint8_t SW_mode;
volatile uint8_t SW_B4;

//PIN変化割り込み
ISR (PCINT1_vect)
{
    //wakeup & read key-sw
	SW_in=(PINC & 0x1f);	//PCINT8～12読み取り
}


//LED flicker.
void FLICKx(uint8_t ix)
{
	uint8_t i;
	for(i=0;i<ix;i++)
	{
		PORTC |= _BV(PC5);  //LED消灯
		_delay_ms(50);
		PORTC &= ~_BV(PC5); //LED点灯
		_delay_ms(50);
	}
}

int main(void)
{
	
	//ポート設定
	DDRB=0x3f; PORTB=0x00;					// --ooooOO　PB1(OC1A:Timer1 out)
	DDRC=0x20; PORTC=0x1F;					// --ORRRRR	(SW入力PCIN8～12:P-UP)
	DDRD=0xfe; PORTD=0x01;					// oooooooR（UART　Rx:P-UP）
	//ピン変化割り込み設定
	PCICR=0x02;								//PCINT1 許可
	PCMSK1=0x1f;							//PCINT8～12 許可
	
	wdt_disable();		//WDT 停止(省電力のため)

	PRR=0xe7;			//電力削減レジスタ(Timer1のみ起動)
	set_sleep_mode (SLEEP_MODE_PWR_DOWN); //CLK全停止省電力モード
	sleep_enable();		//スリープ許可
	
	sei();				//割り込み許可
	
	FLICKx(3);			//起動確認：LED点灯x3


	for(;;)
	{

		PRR=0xef;					//電力削減レジスタ(Timer1も停止)
		sleep_cpu();				//CPUアイドルスリープ

		//SW入力(割り込みでWakeUp)
		
		// Push SW ON
		if((SW_in & 0x10) == 0x00){
			countup();
		}
		else{
			switch(SW_in & 0x07)
			{
				case(0x07):	//SWなし
				case(0x06):	//SW1(Input)
					SW_jdg=0x03;  SW_B4=0x04;
				break;
				case(0x04):	//SW1&2(Input)
				case(0x02):	//SW1&3(Input)
				case(0x00):	//SW1&2&3(Input)
					SW_jdg=0x03;
				break;

				case(0x05):	//SW2(Input)
					SW_jdg=0x01;  SW_B4=0x02;
				break;

				case(0x01):	//SW2&3(Input)
				SW_jdg=0x01; 
				break;

				case(0x03):	//SW3(Input)
					SW_jdg=0x00; SW_B4=0x01;
				break;

				default:	//判定なし
				break;
			}

			if (SW_B4 == SW_jdg){
			 if((SW_in & 0x08) != 0x00){ // HPD_SINK UP
				countup();
			 }
			}
			else{
				SW_mode = SW_jdg;
			}
			SW_B4 = SW_jdg;
		
		}

	FLICKx(1);			//起動確認：LED点灯x1
	PORTB = SW_mode;	//Mode OutPut

	}
}

// 1→2→3→1 Round CountUp.
void countup(void)
{
	if(SW_mode==0x00) {SW_mode=0x03;}
	else{
		if(SW_mode==0x01){ SW_mode=0x00;}
		else{
			SW_mode=0x01;
		}
	}
}
